@@ -74,7 +74,7 @@ exactPrintWithOptions r ast as =
 data PrintOptions m a = PrintOptions
             {
               epAnn :: !Annotation
-#if __GLASGOW_HASKELL__ > 806
+#if (__GLASGOW_HASKELL__ > 806) && (__GLASGOW_HASKELL__ < 900)
             , epAstPrint :: forall ast . (Data ast, GHC.HasSrcSpan ast) => ast -> a -> m a
 #else
             , epAstPrint :: forall ast . Data ast => GHC.Located ast -> a -> m a
@@ -87,7 +87,7 @@ data PrintOptions m a = PrintOptions
 
 -- | Helper to create a 'PrintOptions'
 printOptions ::
-#if __GLASGOW_HASKELL__ > 806
+#if (__GLASGOW_HASKELL__ > 806) && (__GLASGOW_HASKELL__ < 900)
       (forall ast . (Data ast, GHC.HasSrcSpan ast) => ast -> a -> m a)
 #else
       (forall ast . Data ast => GHC.Located ast -> a -> m a)
@@ -165,7 +165,11 @@ printInterpret m = iterTM go (hoistFreeT (return . runIdentity) m)
   where
     go :: AnnotationF (EP w m a) -> EP w m a
     go (MarkEOF next) =
+#if __GLASGOW_HASKELL__ >= 900
+      printStringAtMaybeAnn        AnnEofPos  (Just "") >> next
+#else
       printStringAtMaybeAnn (G GHC.AnnEofPos) (Just "") >> next
+#endif
     go (MarkPrim kwid mstr next) =
       markPrim (G kwid) mstr >> next
     go (MarkPPOptional kwid mstr next) =
@@ -240,7 +244,7 @@ printStoredString = do
     _                    -> return ()
 #endif
 
-withSortKey :: (Monad m, Monoid w) => [(GHC.SrcSpan, Annotated ())] -> EP w m ()
+withSortKey :: (Monad m, Monoid w) => [(AnnSpan, Annotated ())] -> EP w m ()
 withSortKey xs = do
   Ann{..} <- asks epAnn
   let ordered = case annSortKey of
@@ -253,7 +257,7 @@ withSortKey xs = do
                                          )
   mapM_ (printInterpret . snd) ordered
 
-withSortKeyContexts :: (Monad m, Monoid w) => ListContexts -> [(GHC.SrcSpan, Annotated ())] -> EP w m ()
+withSortKeyContexts :: (Monad m, Monoid w) => ListContexts -> [(AnnSpan, Annotated ())] -> EP w m ()
 withSortKeyContexts ctxts xs = do
   Ann{..} <- asks epAnn
   let ordered = case annSortKey of
@@ -290,7 +294,7 @@ allAnns kwid = printStringAtMaybeAnnAll (G kwid) Nothing
 -- |First move to the given location, then call exactP
 -- exactPC :: (Data ast, Monad m, Monoid w) => GHC.Located ast -> EP w m a -> EP w m a
 -- exactPC :: (Data ast, Data (GHC.SrcSpanLess ast), GHC.HasSrcSpan ast, Monad m, Monoid w)
-#if __GLASGOW_HASKELL__ > 806
+#if (__GLASGOW_HASKELL__ > 806) && (__GLASGOW_HASKELL__ < 900)
 exactPC :: (Data ast, Data (GHC.SrcSpanLess ast), GHC.HasSrcSpan ast, Monad m, Monoid w)
         => ast -> EP w m a -> EP w m a
 #else
@@ -328,7 +332,7 @@ advance cl = do
   colOffset <- getLayoutOffset
   printWhitespace (undelta p cl colOffset)
 
-#if __GLASGOW_HASKELL__ > 806
+#if (__GLASGOW_HASKELL__ > 806) && (__GLASGOW_HASKELL__ < 900)
 getAndRemoveAnnotation :: (Monad m, Monoid w, Data a, Data (GHC.SrcSpanLess a), GHC.HasSrcSpan a)
                        => a -> EP w m (Maybe Annotation)
 #else
