@@ -7,21 +7,30 @@ module Test.Transform where
 import Language.Haskell.GHC.ExactPrint
 import Language.Haskell.GHC.ExactPrint.Types
 import Language.Haskell.GHC.ExactPrint.Parsers
+import Language.Haskell.GHC.ExactPrint.Utils
 
+#if __GLASGOW_HASKELL__ >= 900
+import qualified GHC                       as GHC
+import qualified GHC.Data.Bag              as GHC
+import qualified GHC.Data.FastString       as GHC
+import qualified GHC.Types.Name.Occurrence as GHC
+import qualified GHC.Types.Name.Reader     as GHC
+import qualified GHC.Types.SrcLoc          as GHC
+#else
 import qualified Bag            as GHC
 import qualified GHC            as GHC
 import qualified OccName        as GHC
 import qualified RdrName        as GHC
 import qualified SrcLoc         as GHC
 import qualified FastString     as GHC
+#endif
 
 import qualified Data.Generics as SYB
--- import qualified GHC.SYB.Utils as SYB
 
 import System.FilePath
 import qualified Data.Map as Map
 -- import Data.List
-import Data.Maybe
+-- import Data.Maybe
 
 import Test.Common
 
@@ -72,7 +81,7 @@ mkTestMod suffix dir f fp =
       expected       = basename <.> suffix
       writeFailure   = writeFile (basename <.> "out")
   in
-    TestCase (do r <- either (\(ParseFailure _ s) -> error (s ++ basename)) id
+    TestCase (do r <- either (\(ParseFailure s) -> error (s ++ basename)) id
                         <$> genTest f basename expected
                  writeFailure (debugTxt r)
                  assertBool fp (status r == Success))
@@ -123,7 +132,7 @@ changeLocalDecls2 ans (GHC.L l p) = do
 #endif
         newSpan <- uniqueSrcSpanT
         let
-          newAnnKey = AnnKey newSpan (CN "HsValBinds")
+          newAnnKey = AnnKey (rs newSpan) (CN "HsValBinds")
           addWhere mkds =
             case Map.lookup (mkAnnKey m) mkds of
               Nothing -> error "wtf"
@@ -131,7 +140,7 @@ changeLocalDecls2 ans (GHC.L l p) = do
                 where
                   ann1 = ann { annsDP = annsDP ann ++ [(G GHC.AnnWhere,DP (1,2))]
                              , annCapturedSpan = Just newAnnKey
-                             , annSortKey = Just [ls, ld]
+                             , annSortKey = Just [rs ls, rs ld]
                              }
                   mkds2 = Map.insert (mkAnnKey m) ann1 mkds
                   ann2 = annNone
@@ -141,7 +150,7 @@ changeLocalDecls2 ans (GHC.L l p) = do
         -- logTr $ "(m,decls)=" ++ show (mkAnnKey m,map mkAnnKey decls)
         modifyAnnsT (captureOrderAnnKey newAnnKey decls)
 #if __GLASGOW_HASKELL__ > 804
-        let binds = (GHC.HsValBinds GHC.noExt (GHC.ValBinds GHC.noExt (GHC.listToBag $ [GHC.L ld decl])
+        let binds = (GHC.HsValBinds noExt (GHC.ValBinds noExt (GHC.listToBag $ [GHC.L ld decl])
                                     [GHC.L ls sig]))
 #else
         let binds = (GHC.HsValBinds (GHC.ValBindsIn (GHC.listToBag $ [GHC.L ld decl])
@@ -157,7 +166,7 @@ changeLocalDecls2 ans (GHC.L l p) = do
         return (GHC.L lm (GHC.Match mln pats (GHC.GRHSs rhs (GHC.L bindSpan binds))))
 #else
         bindSpan <- uniqueSrcSpanT
-        return (GHC.L lm (GHC.Match GHC.noExt mln pats (GHC.GRHSs GHC.noExt rhs (GHC.L bindSpan binds))))
+        return (GHC.L lm (GHC.Match noExt mln pats (GHC.GRHSs noExt rhs (GHC.L bindSpan binds))))
 #endif
       replaceLocalBinds x = return x
   -- putStrLn $ "log:" ++ intercalate "\n" w
@@ -205,8 +214,8 @@ changeLocalDecls ans (GHC.L l p) = do
         -- logTr $ "(m,decls)=" ++ show (mkAnnKey m,map mkAnnKey decls)
         modifyAnnsT (captureOrder m decls)
 #if __GLASGOW_HASKELL__ > 804
-        let binds' = (GHC.HsValBinds GHC.noExt
-                          (GHC.ValBinds GHC.noExt (GHC.listToBag $ (GHC.L ld decl):GHC.bagToList binds)
+        let binds' = (GHC.HsValBinds noExt
+                          (GHC.ValBinds noExt (GHC.listToBag $ (GHC.L ld decl):GHC.bagToList binds)
                                           (GHC.L ls sig:sigs)))
 #else
         let binds' = (GHC.HsValBinds
@@ -220,7 +229,7 @@ changeLocalDecls ans (GHC.L l p) = do
 #elif __GLASGOW_HASKELL__ <= 804
         return (GHC.L lm (GHC.Match mln pats (GHC.GRHSs rhs (GHC.L lb binds'))))
 #else
-        return (GHC.L lm (GHC.Match GHC.noExt mln pats (GHC.GRHSs GHC.noExt rhs (GHC.L lb binds'))))
+        return (GHC.L lm (GHC.Match noExt mln pats (GHC.GRHSs noExt rhs (GHC.L lb binds'))))
 #endif
       replaceLocalBinds x = return x
   -- putStrLn $ "log:" ++ intercalate "\n" w
@@ -251,19 +260,28 @@ changeWhereIn3 declIndex ans p = return (ans',p')
     (p',(ans',_),_) = runTransform ans doTransform
     doTransform = doRmDecl p
 
-    doRmDecl (GHC.L l (GHC.HsModule mmn mexp imps decls mdepr haddock)) = do
+#if __GLASGOW_HASKELL__ >= 900
+    doRmDecl (GHC.L l (GHC.HsModule lo mmn mexp imps decls mdepr haddock)) = do
+#else
+    doRmDecl (GHC.L l (GHC.HsModule    mmn mexp imps decls mdepr haddock)) = do
+#endif
       let
         -- declIndex = 2 -- zero based
         decls1 = take declIndex decls
         decls2 = drop (declIndex + 1) decls
         decls' = decls1 ++ decls2
-      return (GHC.L l (GHC.HsModule mmn mexp imps decls' mdepr haddock))
+#if __GLASGOW_HASKELL__ >= 900
+      return (GHC.L l (GHC.HsModule lo mmn mexp imps decls' mdepr haddock))
+#else
+      return (GHC.L l (GHC.HsModule    mmn mexp imps decls' mdepr haddock))
+#endif
       -- error $ "doRmDecl:decls2=" ++ showGhc (length decls,decls1,decls2)
 
 -- ---------------------------------------------------------------------
 
 changeRenameCase1 :: Changer
 changeRenameCase1 ans parsed = return (ans,rename "bazLonger" [((3,15),(3,18))] parsed)
+-- changeRenameCase1 ans parsed = return (ans,rename "bazLonger" [((3,15),(3,17))] parsed)
 
 changeRenameCase2 :: Changer
 changeRenameCase2 ans parsed = return (ans,rename "fooLonger" [((3,1),(3,4))] parsed)
@@ -307,12 +325,7 @@ rename newNameStr spans a
     newName = GHC.mkRdrUnqual (GHC.mkVarOcc newNameStr)
 
     cond :: GHC.SrcSpan -> Bool
-    cond ln = ln `elem` srcSpans
-      where
-        srcSpans  = map (\(start, end) -> GHC.mkSrcSpan (f start) (f end)) spans
-        fname = fromMaybe (GHC.mkFastString "f") (GHC.srcSpanFileName_maybe ln)
-        f = uncurry (GHC.mkSrcLoc fname)
-
+    cond ln = ss2range ln `elem` spans
 
     replaceRdr :: GHC.Located GHC.RdrName -> GHC.Located GHC.RdrName
     replaceRdr (GHC.L ln _)
@@ -326,20 +339,20 @@ rename newNameStr spans a
 #elif __GLASGOW_HASKELL__ <= 804
         | cond ln = GHC.L ln (GHC.HsVar (GHC.L ln newName))
 #else
-        | cond ln = GHC.L ln (GHC.HsVar GHC.noExt (GHC.L ln newName))
+        | cond ln = GHC.L ln (GHC.HsVar noExt (GHC.L ln newName))
 #endif
     replaceHsVar x = x
 
 
 
-#if __GLASGOW_HASKELL__ > 806
+#if (__GLASGOW_HASKELL__ > 806) && (__GLASGOW_HASKELL__ < 900)
     replacePat :: GHC.LPat GhcPs -> GHC.LPat GhcPs
     replacePat (GHC.dL->GHC.L ln (GHC.VarPat {}))
-        | cond ln = GHC.cL ln (GHC.VarPat GHC.noExt (GHC.cL ln newName))
+        | cond ln = GHC.cL ln (GHC.VarPat noExt (GHC.cL ln newName))
 #elif __GLASGOW_HASKELL__ > 804
     replacePat :: GHC.LPat GhcPs -> GHC.LPat GhcPs
     replacePat (GHC.L ln (GHC.VarPat {}))
-        | cond ln = GHC.L ln (GHC.VarPat GHC.noExt (GHC.L ln newName))
+        | cond ln = GHC.L ln (GHC.VarPat noExt (GHC.L ln newName))
 #elif __GLASGOW_HASKELL__ > 802
     replacePat :: GHC.LPat GhcPs -> GHC.LPat GhcPs
     replacePat (GHC.L ln (GHC.VarPat {}))
@@ -364,7 +377,7 @@ rename newNameStr spans a
 -- #elif __GLASGOW_HASKELL__ <= 804
 --         | cond ln = GHC.L ln (GHC.VarPat (GHC.L ln newName))
 -- #else
---         | cond ln = GHC.L ln (GHC.VarPat GHC.noExt (GHC.L ln newName))
+--         | cond ln = GHC.L ln (GHC.VarPat noExt (GHC.L ln newName))
 -- #endif
 --     replacePat x = x
 
@@ -378,11 +391,7 @@ changeWhereIn4 ans parsed
   where
     replace :: GHC.Located GHC.RdrName -> GHC.Located GHC.RdrName
     replace (GHC.L ln _n)
-      | ln == (g (12,16) (12,17)) = GHC.L ln (GHC.mkRdrUnqual (GHC.mkVarOcc "p_2"))
-      where
-        g start end = GHC.mkSrcSpan (f start) (f end)
-        fname = fromMaybe (GHC.mkFastString "f") (GHC.srcSpanFileName_maybe ln)
-        f = uncurry (GHC.mkSrcLoc fname)
+      | ss2range ln == ((12,16),(12,17)) = GHC.L ln (GHC.mkRdrUnqual (GHC.mkVarOcc "p_2"))
     replace x = x
 
 -- ---------------------------------------------------------------------
@@ -414,7 +423,7 @@ changeLetIn1 ans parsed
 #elif __GLASGOW_HASKELL__ <= 804
          in (GHC.HsLet (GHC.L lb (GHC.HsValBinds (GHC.ValBindsIn bagDecls' sigs))) expr)
 #else
-         in (GHC.HsLet GHC.noExt (GHC.L lb (GHC.HsValBinds x (GHC.ValBinds xv bagDecls' sigs))) expr)
+         in (GHC.HsLet noExt (GHC.L lb (GHC.HsValBinds x (GHC.ValBinds xv bagDecls' sigs))) expr)
 #endif
 
     replace x = x
@@ -563,12 +572,16 @@ addLocaLDecl6 ans lp = do
         [d1,d2] <- hsDecls lp
         balanceComments d1 d2
 
-#if __GLASGOW_HASKELL__ <= 710
-        let GHC.L _ (GHC.ValD (GHC.FunBind  _ _ (GHC.MG [m1,m2] _ _ _) _ _ _)) = d1
-#elif __GLASGOW_HASKELL__ <= 804
+#if __GLASGOW_HASKELL__ >= 900
+        let GHC.L _ (GHC.ValD _ (GHC.FunBind _ _ (GHC.MG _ (GHC.L _ [m1,m2]) _) _)) = d1
+#elif __GLASGOW_HASKELL__ > 808
+        let GHC.L _ (GHC.ValD _ (GHC.FunBind _ _ (GHC.MG _ (GHC.L _ [m1,m2]) _) _ _)) = d1
+#elif __GLASGOW_HASKELL__ > 804
+        let GHC.L _ (GHC.ValD _ (GHC.FunBind _ _ (GHC.MG _ (GHC.L _ [m1,m2]) _) _ _)) = d1
+#elif __GLASGOW_HASKELL__ > 710
         let GHC.L _ (GHC.ValD (GHC.FunBind  _ (GHC.MG (GHC.L _ [m1,m2]) _ _ _) _ _ _)) = d1
 #else
-        let GHC.L _ (GHC.ValD _ (GHC.FunBind _ _ (GHC.MG _ (GHC.L _ [m1,m2]) _) _ _)) = d1
+        let GHC.L _ (GHC.ValD (GHC.FunBind  _ _ (GHC.MG [m1,m2] _ _ _) _ _ _)) = d1
 #endif
         balanceComments m1 m2
 
@@ -688,7 +701,7 @@ rmDecl5 ans lp = do
 #elif __GLASGOW_HASKELL__ <= 804
             return (GHC.HsLet (GHC.L l lb') expr)
 #else
-            return (GHC.HsLet GHC.noExt (GHC.L l lb') expr)
+            return (GHC.HsLet noExt (GHC.L l lb') expr)
 #endif
           go x = return x
 
@@ -790,8 +803,8 @@ addHiding1 ans (GHC.L l p) = do
           n1 = GHC.L l1 (GHC.mkVarUnqual (GHC.mkFastString "n1"))
           n2 = GHC.L l2 (GHC.mkVarUnqual (GHC.mkFastString "n2"))
 #if __GLASGOW_HASKELL__ > 804
-          v1 = GHC.L l1 (GHC.IEVar GHC.noExt (GHC.L l1 (GHC.IEName n1)))
-          v2 = GHC.L l2 (GHC.IEVar GHC.noExt (GHC.L l2 (GHC.IEName n2)))
+          v1 = GHC.L l1 (GHC.IEVar noExt (GHC.L l1 (GHC.IEName n1)))
+          v2 = GHC.L l2 (GHC.IEVar noExt (GHC.L l2 (GHC.IEName n2)))
 #elif __GLASGOW_HASKELL__ > 800
           v1 = GHC.L l1 (GHC.IEVar (GHC.L l1 (GHC.IEName n1)))
           v2 = GHC.L l2 (GHC.IEVar (GHC.L l2 (GHC.IEName n2)))
@@ -824,8 +837,8 @@ addHiding2 ans (GHC.L l p) = do
           n1 = GHC.L l1 (GHC.mkVarUnqual (GHC.mkFastString "n1"))
           n2 = GHC.L l2 (GHC.mkVarUnqual (GHC.mkFastString "n2"))
 #if __GLASGOW_HASKELL__ > 804
-          v1 = GHC.L l1 (GHC.IEVar GHC.noExt (GHC.L l1 (GHC.IEName n1)))
-          v2 = GHC.L l2 (GHC.IEVar GHC.noExt (GHC.L l2 (GHC.IEName n2)))
+          v1 = GHC.L l1 (GHC.IEVar noExt (GHC.L l1 (GHC.IEName n1)))
+          v2 = GHC.L l2 (GHC.IEVar noExt (GHC.L l2 (GHC.IEName n2)))
 #elif __GLASGOW_HASKELL__ > 800
           v1 = GHC.L l1 (GHC.IEVar (GHC.L l1 (GHC.IEName n1)))
           v2 = GHC.L l2 (GHC.IEVar (GHC.L l2 (GHC.IEName n2)))
